; $Id: wisard_data2mdata.pro,v 1.6 2007/02/02 16:35:39 meimon Exp $ 
; This file is part of the WISARD software.
;
; Copyright (c) S. Meimon and L. Mugnier, ONERA, 2004-2007.
;
; This software, WISARD, is made of wisard.pro and all the wisard_*.pro files
; under the same directory, as well as the example batch file wisard_batch.pro
; 
; This software is copyright ONERA.
; The authors are Serge Meimon and Laurent Mugnier.
; E-mail: meimon at onera.fr and mugnier at onera.fr
;
; This software is a computer program whose purpose is to reconstruct an image
; of an observed object from a set of interferometric measurements.
;
; This software is governed by the CeCILL-B license under French law and
; abiding by the rules of distribution of free software. You can use, modify
; and/ or redistribute the software under the terms of the CeCILL-B license as
; circulated by CEA, CNRS and INRIA at the following URL
; "http://www.cecill.info".
;
; As a counterpart to the access to the source code and  rights to copy,
; modify and redistribute granted by the license, users are provided only
; with a limited warranty  and the software's author,  the holder of the
; economic rights,  and the successive licensors  have only  limited
; liability. 
;
; In this respect, the user's attention is drawn to the risks associated
; with loading,  using,  modifying and/or developing or reproducing the
; software by the user in light of its specific status of free software,
; that may mean  that it is complicated to manipulate,  and  that  also
; therefore means  that it is reserved for developers  and  experienced
; professionals having in-depth computer knowledge. Users are therefore
; encouraged to load and test the software's suitability as regards their
; requirements in conditions enabling the security of their systems and/or 
; data to be ensured and,  more generally, to use and operate it in the 
; same conditions as regards security. 
;
; The fact that you are presently reading this means that you have had
; knowledge of the CeCILL-B license and that you accept its terms.
;
; Additionally, if you use this code or a code derived from it (e.g., for a
; publication), please cite the following papers, on which it is based:
;   
; (1) S. Meimon, L. M. Mugnier, and G. Le Besnerais,
;   "Reconstruction method for weak-phase optical interferometry", 
;   Opt. Lett., 30(14):1809-1811, July 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-OL-05.pdf
; (2) S. Meimon, L. M. Mugnier, and Guy Le Besnerais,
;   "A convex approximation of the likelihood in optical interferometry",
;   J. Opt. Soc. Am. A, November 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-JOSAA-05.pdf
;

;+
; NAME:
;	WISARD_DATA2MDATA - Computes myopic data from input data
;
; CATEGORY:
;	Array and Image Processing Routines 
;
; CALLING SEQUENCE:
;	mdata = WISARD_DATA2MDATA( data, OPERATORS = operators[, /VERSION] [, /HELP])
;
; PURPOSE:
;   This function computes and returns myopic data from input
;   data. The myopic data are aranged in a array of Structures
;   containing the following fields (see doc scientifique):
;	 VISAMP          DOUBLE    Array[N_BASES]
;    VISAMPERR       DOUBLE    Array[N_BASES]
;    VISPHI          DOUBLE    Array[N_BASES]
;    VISPHIERR       DOUBLE    Array[N_BASES]
;    FREQS_U         DOUBLE    Array[N_BASES]
;    FREQS_V         DOUBLE    Array[N_BASES]
;
;
; POSITIONAL PARAMETERS:
;	data: (input)Array of Structures containing the following fields (see doc scientifique): 
;    VIS2            DOUBLE    Array[N_BASES]
;    VIS2ERR         DOUBLE    Array[N_BASES]
;    CLOT            DOUBLE    Array[N_CLOTS]
;    CLOTERR         DOUBLE    Array[N_CLOTS]
;    FREQS_U         DOUBLE    Array[N_TELS-1]
;    FREQS_V         DOUBLE    Array[N_TELS-1]
;	
; KEYWORD PARAMETERS:
;
;   /VERSION : (optional input) prints version number before execution.
;   
;   /HELP    : (optional input) prints the documentation and exits.
;
;
; AUTHORS:
; Serge Meimon and Laurent Mugnier (ONERA). 
; Contact: lastname at onera.fr
;
;
; RESTRICTIONS:
;   This code is copyright (c) S. Meimon and L. Mugnier, ONERA, 2004-2007. 
;   
;   WISARD is governed by the CeCILL-B license under French law and abiding by
;   the rules of distribution of free software. You can use, modify and/ or
;   redistribute the software under the terms of the CeCILL-B license as
;   circulated by CEA, CNRS and INRIA at the following URL:
;   "http://www.cecill.info".
;   See source code for the full notice.
;
;   Additionally, if you use this code or a code derived from it (e.g., for a
;   publication), please cite the following papers, on which it is based:
;   
;   (1) S. Meimon, L. M. Mugnier, and G. Le Besnerais,
;   "Reconstruction method for weak-phase optical interferometry", 
;   Opt. Lett., 30(14):1809-1811, July 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-OL-05.pdf
;   (2) S. Meimon, L. M. Mugnier, and Guy Le Besnerais,
;   "A convex approximation of the likelihood in optical interferometry",
;   J. Opt. Soc. Am. A, November 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-JOSAA-05.pdf
;
; EXAMPLE:
;	mdata = WISARD_DATA2MDATA(data, OPERATORS = operators, VERSION = version)
;
; SEE ALSO:
;   All the WISARD_*.pro files, which are part of WISARD and covered by the
;   same copyright and license.
;   
; HISTORY:
; $Log: wisard_data2mdata.pro,v $
; Revision 1.6  2007/02/02 16:35:39  meimon
; translation of french comments in english
;
; Revision 1.5  2007/01/29 13:07:53  meimon
; doc added
;
; Revision 1.4  2007/01/25 14:11:01  meimon
; 5.92->6
; 1.71..->sqrt(3)
;
; Revision 1.3  2007/01/23 16:45:38  mugnier
; - Si     data.vis2 < 0                 on prend mdata.visamp=0
; - Si 0 < data.vis2 < data.vis2err/5.92 on prend mdata.visamp= sqrt(data.vis2err/2.)/1.721
; - Si data.vis2 < data.vis2err          on prend mdata.visamperr=sqrt(data.vis2err)/2
;
;-




FUNCTION WISARD_DATA2MDATA, data, OPERATORS = operators, VERSION = version, HELP = help
on_error,2
IF keyword_set(version) THEN $
    printf, -2, '% '+routine_courante()+': $Revision: 1.6 $, $Date: 2007/02/02 16:35:39 $'

IF (n_params() NE 1) OR keyword_set(help) THEN BEGIN 
    message, 'Help required or incorrect syntax. Documentation:', /INFO
    doc_library, routine_courante()
    retall
ENDIF


n_tels = n_elements(data[0].freqs_u)+1
operators = WISARD_OPERATORS(n_tels, VERSION = version)
n_dat = n_elements(data)
n_bases = operators.n_bases


mdata = replicate({visamp:dblarr(n_bases), visamperr:dblarr(n_bases), $
                   visphi:dblarr(n_bases), visphierr:dblarr(n_bases), $
                   freqs_u:dblarr(n_bases),freqs_v:dblarr(n_bases) }, n_dat)

;;;calcul de visamp
tmp = where(data.vis2+data.vis2err LT 0, count); s< -sigma_s < 0  => erreur
IF count NE 0 THEN message, 'There are squared visibility data lower than -sigma'
mdata.visamp = sqrt(data.vis2 > 0D)
;mdata.visamp = sqrt(abs(data.vis2))

;;;calcul de visamperr
mdata.visamperr = data.vis2err/(2.*sqrt(abs(data.vis2)))

ind_small_s = where(data.vis2 - data.vis2err LT 0, count) ;s<sigma_s
ind_vsmall_s = where((data.vis2 - data.vis2err/6 LT 0) AND (data.vis2 GT 0D) , count2) ;s<sigma_s/6
IF count NE 0 THEN BEGIN 
   print, 'There are ', count, ' squared visibilities among ', n_dat*n_bases, $
          'lower than sigma'
 
   mdata.visamperr[ind_small_s] = sqrt(data.vis2err[ind_small_s])/(2.)
   
ENDIF
IF count2 NE 0 THEN BEGIN 
  print, 'There are ', count2, ' squared visibilities among ', n_dat*n_bases, $
         '>0 and lower than sigma/6'
  mdata.visamp[ind_vsmall_s] = sqrt(data.vis2err[ind_vsmall_s]/6.)
ENDIF

;;;calcul de visphi
mdata.visphi = operators.dagC#data.clot

;;;calcul de visphierr
mdata.visphierr = 3.*((operators.dagC)^2#data.cloterr)

;;;calcul de freqs
mdata.freqs_u = operators._B#data.freqs_u
mdata.freqs_v = operators._B#data.freqs_v



return,  mdata
END
seed = 1
data0 = {vis2:abs(randomn(seed, 15))*10., vis2err:abs(randomn(seed, 15)), clot:randomn(seed,10), $
         cloterr:abs(randomn(seed, 10)), freqs_u:randomn(seed, 5), freqs_v:randomn(seed, 5)}
ndat = 1000
data=replicate(data0,ndat)
mdata = WISARD_DATA2MDATA(data)
END

