; $Id: wisard_jtotal_x.pro,v 1.3 2007/01/29 10:41:27 meimon Exp $ 
; This file is part of the WISARD software.
;
; Copyright (c) S. Meimon and L. Mugnier, ONERA, 2004-2007.
;
; This software, WISARD, is made of wisard.pro and all the wisard_*.pro files
; under the same directory, as well as the example batch file wisard_batch.pro
; 
; This software is copyright ONERA.
; The authors are Serge Meimon and Laurent Mugnier.
; E-mail: meimon at onera.fr and mugnier at onera.fr
;
; This software is a computer program whose purpose is to reconstruct an image
; of an observed object from a set of interferometric measurements.
;
; This software is governed by the CeCILL-B license under French law and
; abiding by the rules of distribution of free software. You can use, modify
; and/ or redistribute the software under the terms of the CeCILL-B license as
; circulated by CEA, CNRS and INRIA at the following URL
; "http://www.cecill.info".
;
; As a counterpart to the access to the source code and  rights to copy,
; modify and redistribute granted by the license, users are provided only
; with a limited warranty  and the software's author,  the holder of the
; economic rights,  and the successive licensors  have only  limited
; liability. 
;
; In this respect, the user's attention is drawn to the risks associated
; with loading,  using,  modifying and/or developing or reproducing the
; software by the user in light of its specific status of free software,
; that may mean  that it is complicated to manipulate,  and  that  also
; therefore means  that it is reserved for developers  and  experienced
; professionals having in-depth computer knowledge. Users are therefore
; encouraged to load and test the software's suitability as regards their
; requirements in conditions enabling the security of their systems and/or 
; data to be ensured and,  more generally, to use and operate it in the 
; same conditions as regards security. 
;
; The fact that you are presently reading this means that you have had
; knowledge of the CeCILL-B license and that you accept its terms.
;
; Additionally, if you use this code or a code derived from it (e.g., for a
; publication), please cite the following papers, on which it is based:
;   
; (1) S. Meimon, L. M. Mugnier, and G. Le Besnerais,
;   "Reconstruction method for weak-phase optical interferometry", 
;   Opt. Lett., 30(14):1809-1811, July 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-OL-05.pdf
; (2) S. Meimon, L. M. Mugnier, and Guy Le Besnerais,
;   "A convex approximation of the likelihood in optical interferometry",
;   J. Opt. Soc. Am. A, November 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-JOSAA-05.pdf
;

;+
; NAME:
;	WISARD_JTOTAL_X - computes the criterion (data + prior) to minimize and its gradient
;                         w.r.t. the object
;
; CATEGORY:
;	Array and Image Processing Routines 
;
; CALLING SEQUENCE:
;
; value = WISARD_JTOTAL_X( x, gradient,WEIGHTS_CONSTANT = weights_constant, $
;                          WEIGHTS_ALPHA = weights_alpha, PRIOR = prior,
;                          CRIT_ARRAY = crit_array , OPERATORS = operators,
;                          [, /VERSION] [, /HELP])
;
; POSITIONAL PARAMETERS:
;	x:                (input ) current object
;	gradient:         (optional input/output ) if set, the gradient is
;                     computed and stored in this keyword
;
;	
; KEYWORD PARAMETERS:
;
;  _BALPHA:           (input ) current differential baseline aberrations (see
;                     WISARD_OPERATORS.PRO)
;  WEIGHTS_CONSTANT:  (input ) see WISARD.PRO
;  WEIGHTS_ALPHA:     (input ) see WISARD.PRO
;  PRIOR:             (input ) see WISARD_SET_REGUL.PRO. Contains all the
;                     informations on the regularization used.
;  CRIT_ARRAY:        (input/output) contains the current value of the
;                                    criterion and the value of the
;                                    regularization terms 
;  OPERATORS:         (input ) see WISARD_OPERATORS.PRO
;  /VERSION : (optional input) prints version number before execution.
;   
;  /HELP    : (optional input) prints the documentation and exits.
;
;
; AUTHORS:
; Serge Meimon and Laurent Mugnier (ONERA). 
; Contact: lastname at onera.fr
;
;
; RESTRICTIONS:
;   This code is copyright (c) S. Meimon and L. Mugnier, ONERA, 2004-2007. 
;   
;   WISARD is governed by the CeCILL-B license under French law and abiding by
;   the rules of distribution of free software. You can use, modify and/ or
;   redistribute the software under the terms of the CeCILL-B license as
;   circulated by CEA, CNRS and INRIA at the following URL:
;   "http://www.cecill.info".
;   See source code for the full notice.
;
;   Additionally, if you use this code or a code derived from it (e.g., for a
;   publication), please cite the following papers, on which it is based:
;   
;   (1) S. Meimon, L. M. Mugnier, and G. Le Besnerais,
;   "Reconstruction method for weak-phase optical interferometry", 
;   Opt. Lett., 30(14):1809-1811, July 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-OL-05.pdf
;   (2) S. Meimon, L. M. Mugnier, and Guy Le Besnerais,
;   "A convex approximation of the likelihood in optical interferometry",
;   J. Opt. Soc. Am. A, November 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-JOSAA-05.pdf
;
; EXAMPLE:
;	Please provide a simple example here. 
;
; SEE ALSO:
;   All the WISARD_*.pro files, which are part of WISARD and covered by the
;   same copyright and license.
;   
; HISTORY:
;   $Log: wisard_jtotal_x.pro,v $
;   Revision 1.3  2007/01/29 10:41:27  meimon
;   *** empty log message ***
;
;   Revision 1.2  2007/01/29 10:40:04  meimon
;   *** empty log message ***
;
;-


FUNCTION WISARD_JTOTAL_X, x, gradient,WEIGHTS_CONSTANT = weights_constant, WEIGHTS_ALPHA = weights_alpha $
                          , PRIOR = prior, CRIT_ARRAY = crit_array $
                          , OPERATORS = operators, VERSION = version, HELP $
                              = help
on_error,2
IF keyword_set(version) THEN $
    printf, -2, '% '+routine_courante()+': $Revision: 1.3 $, $Date: 2007/01/29 10:41:27 $'

IF (n_params() GT 2) OR keyword_set(help) THEN BEGIN ;(fill-in from syntax)
    message, 'Help required or incorrect syntax. Documentation:', /INFO
    doc_library, routine_courante()
    retall
ENDIF






;;%%%%%%%normalisation de l'objet
factor = 1D/total(x)
norm_x = x*factor
;;;;;;;;

IF (n_params() EQ 2) THEN BEGIN 
    gradient_data = 1D & gradient_prior = 1D & norm_gradient_prior = 1D
ENDIF ELSE BEGIN
    gradient_data = 0D &  gradient_prior = 0D & norm_gradient_prior = 0D
ENDELSE

crit_array[0] = WISARD_JDATA_X(norm_x,OPERATORS = operators, $
                         RE_PH = weights_alpha.re_ph, IM_PH = weights_alpha.im_ph, $
                         RE_Y_DATA = weights_constant.re_y_data, IM_Y_DATA = weights_constant.im_y_data, $
                         W11 = weights_constant.w11, W12 = weights_constant.w12, W22 = weights_constant.w22, $
                         GRADIENT_X=gradient_data, $
                         VERSION = version)

;;;prvoir les criteres a priori : 


IF (prior.type EQ 'l1l2') THEN BEGIN 
   crit_array[1] = J_PRIOR_L1L2(reform(norm_x, prior.NP,prior.NP) , GRADIENT_O=gradient_prior, $
                                SCALE=prior.scale, DELTA=prior.delta, $
                                WHITE=prior.white, MEAN_O = prior.mean_o, $
                                VERSION = version) 

   IF (n_params() EQ 2) THEN norm_gradient_prior = reform(gradient_prior, prior.squareNP)
ENDIF ELSE IF (prior.type EQ 'psd') THEN BEGIN ; envisager ft_object pour J_PRIOR_GAUSS ?
   crit_array[1] = J_PRIOR_GAUSS(reform(norm_x, prior.NP,prior.NP) , GRADIENT_O = gradient_prior, $
                                 PSD=prior.psd, FT_MEAN_O = $ 
                                 prior.ft_mean_o, VERSION = version) 
   IF (n_params() EQ 2) THEN norm_gradient_prior = reform(gradient_prior, prior.squareNP)
ENDIF ELSE IF (prior.type EQ 'support') THEN BEGIN ; envisager ft_object pour J_PRIOR_GAUSS ?
   crit_array[1] = J_SUPPORT(reform(norm_x, prior.NP,prior.NP) , GRADIENT_O = gradient_prior, $
                                 FWHM_PIX = prior.fwhm_pix, MU_SUP = prior.mu_sup, VERSION = version) 
   IF (n_params() EQ 2) THEN norm_gradient_prior = reform(gradient_prior, prior.squareNP)
ENDIF ELSE crit_array[1] = 0D






IF (n_params() EQ 2) THEN BEGIN
   norm_gradient = gradient_data+norm_gradient_prior
   gradient = norm_gradient   ;
   gradient = factor*(norm_gradient-total(norm_x*norm_gradient))
   ;print, 'total grad', total(gradient)
ENDIF

;print, crit_array
return, total(crit_array)
END
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;%%%%%%%cration des donnes et de la regule
filename = '~/idl/BIC/PHASE1/final1.oifits'
data = wisard_oifits2data(filename)
unemas = 1d-3*(!DPi/180D)/3600D ; une mas en rd
fov = 12.*unemas
;seed = 1
;ndat = 1000
; data0 = {vis2:abs(randomn(seed, 15))*10., vis2err:abs(randomn(seed, 15)), clot:randomn(seed,10), $
;          cloterr:abs(randomn(seed, 10)), freqs_u:abs(randomn(seed, 5))+1., freqs_v:abs(randomn(seed, 5))+1.}
; data=replicate(data0,ndat)


mdata = WISARD_DATA2MDATA(data, OPERATORS = operators)
cmdata = WISARD_MDATA2CMDATA(mdata, OPERATORS = operators)


re_y_data = real(cmdata.vis)
im_y_data = imaginary(cmdata.vis)
abs_y_data = abs(cmdata.vis)
arg_y_data = angle(cmdata.vis)
w11 = cmdata.w_rad*abs2(COS(arg_y_data)) $
      +cmdata.w_tan*abs2(SIN(arg_y_data))
w22 = cmdata.w_tan*abs2(COS(arg_y_data)) $
    +cmdata.w_rad*abs2(SIN(arg_y_data))
w12 = (cmdata.w_rad-cmdata.w_tan)* $
    COS(arg_y_data)*SIN(arg_y_data)
weights_constant = {re_y_data:re_y_data, $
                    im_y_data:im_y_data, $
                    abs_y_data:abs_y_data, $
                    arg_y_data:arg_y_data, $
                    w11:w11, w12:w12, w22:w22, $ 
                    w_rad:cmdata.w_rad, $ 
                    w_tan:cmdata.w_tan}

H=wisard_make_h(FREQS_U=reform(cmdata.freqs_u, operators.n_bases*n_elements(mdata)),$ 
                FREQS_V=reform(cmdata.freqs_v, operators.n_bases*n_elements(mdata)), $
                FOV = fov, np_min = 64)
;H = H*1e10; pour le calcul de grad_num
squareNP = (size(H))[2]
NP = sqrt(squareNP)
prior = wisard_set_regul( SUPPORT = support, POSITIVITY = positivity, $
                        PSD = psd, MEAN_O = mean_o, $
                          SCALE = scale, DELTA = delta, WHITE = white, $
                          ACTIVE_SET = active_set, NP = NP, squareNP =squareNP)

prior.type = 'none'

;%%%%%%%cration de l'objet et weights_x



TH = conj(transpose(H))
dirty_map = TH#reform(cmdata.vis,(size(H))[1])
xdm = real(dirty_map) >0
xrm = randomn(seed, (size(H))[2])*10D;real(dirty_map) >0
;aff, rotate(reform(xdm, sqnpix, sqnpix),5)
x = xdm
x = xrm*.1

factor = 1D/total(x)
print,  'factor', factor
norm_x = x*factor
achix = reform(H#norm_x, operators.n_bases, n_elements(cmdata))
abs_hx = abs(achix)
abs2_hx = abs2(achix)
arg_hx = angle(achix)

wx1 = abs2_hx*(cmdata.w_tan-cmdata.w_rad)
wx2 = 2D*abs_hx*abs_y_data*cmdata.w_rad

weights_x = {abs_hx:abs_hx, arg_hx:arg_hx, wx1:wx1, wx2:wx2}
 
;weights_x.abs_hx = weights_x.abs_hx*0.
;%%%%%%%cration de alpha et weights_alpha
alpha = randomn(seed, operators.n_tels-1, n_elements(cmdata))*1D

_Balpha = operators._B#alpha
P = EXP(!dI*reform(_Balpha, operators.n_bases*n_elements(cmdata)))
ph = diag(P)#H
weights_alpha = {re_ph:real(ph), im_ph:imaginary(ph)}
;%%%%%%%




;%%%%%%%%%%%%%%comparaison des criteres en x et en alpha %%%%%%%%%%%
gradient = 0
crit_array = dblarr(2) 
dummix = WISARD_JTOTAL_X( x, gradient,WEIGHTS_CONSTANT = weights_constant, WEIGHTS_ALPHA = weights_alpha $
                          , PRIOR = prior, CRIT_ARRAY = crit_array $
                          , OPERATORS = operators)

dummy = WISARD_JTOTAL_ALPHA(alpha, gradient, _BALPHA = _Balpha,WEIGHTS_CONSTANT = weights_constant, WEIGHTS_X = weights_x $
                              , METHOD = method, CRIT_ARRAY = crit_array $
                              , OPERATORS = operators)



print,  'dummix', dummix
print, 'dummy', dummy
print, dummix/dummy
END
