; $Id: wisard_plot_fit.pro,v 1.6 2007/02/01 13:31:31 meimon Exp $ 
; This file is part of the WISARD software.
;
; Copyright (c) S. Meimon and L. Mugnier, ONERA, 2004-2007.
;
; This software, WISARD, is made of wisard.pro and all the wisard_*.pro files
; under the same directory, as well as the example batch file wisard_batch.pro
; 
; This software is copyright ONERA.
; The authors are Serge Meimon and Laurent Mugnier.
; E-mail: meimon at onera.fr and mugnier at onera.fr
;
; This software is a computer program whose purpose is to reconstruct an image
; of an observed object from a set of interferometric measurements.
;
; This software is governed by the CeCILL-B license under French law and
; abiding by the rules of distribution of free software. You can use, modify
; and/ or redistribute the software under the terms of the CeCILL-B license as
; circulated by CEA, CNRS and INRIA at the following URL
; "http://www.cecill.info".
;
; As a counterpart to the access to the source code and  rights to copy,
; modify and redistribute granted by the license, users are provided only
; with a limited warranty  and the software's author,  the holder of the
; economic rights,  and the successive licensors  have only  limited
; liability. 
;
; In this respect, the user's attention is drawn to the risks associated
; with loading,  using,  modifying and/or developing or reproducing the
; software by the user in light of its specific status of free software,
; that may mean  that it is complicated to manipulate,  and  that  also
; therefore means  that it is reserved for developers  and  experienced
; professionals having in-depth computer knowledge. Users are therefore
; encouraged to load and test the software's suitability as regards their
; requirements in conditions enabling the security of their systems and/or 
; data to be ensured and,  more generally, to use and operate it in the 
; same conditions as regards security. 
;
; The fact that you are presently reading this means that you have had
; knowledge of the CeCILL-B license and that you accept its terms.
;
; Additionally, if you use this code or a code derived from it (e.g., for a
; publication), please cite the following papers, on which it is based:
;   
; (1) S. Meimon, L. M. Mugnier, and G. Le Besnerais,
;   "Reconstruction method for weak-phase optical interferometry", 
;   Opt. Lett., 30(14):1809-1811, July 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-OL-05.pdf
; (2) S. Meimon, L. M. Mugnier, and Guy Le Besnerais,
;   "A convex approximation of the likelihood in optical interferometry",
;   J. Opt. Soc. Am. A, November 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-JOSAA-05.pdf
;

;+
; NAME:
;	WISARD_PLOT_FIT - plots the fitting between data and reconstruction in visibility moduli 
;
; CATEGORY:
;	Array and Image Processing Routines 
;
; CALLING SEQUENCE:
;	WISARD_PLOT_FIT,RAD_FREQS = rad_freqs, ABS_HX = abs_hx, ABS_Y = abs_y, $
;             INV_SIGMA_RAD = inv_sigma_rad[, /VERSION] [, /HELP]
;
;
; PURPOSE:
;   This procedure plots the visibility moduli for the data (abs_y), the model (abs_hx), and the
;residual in moduli (|abs_y-abs_hx|) in tenths of sigma.
;
;
; POSITIONAL PARAMETERS:
;	rad_freqs     : (input) radial frequencies
;	abs_hx        : (input) visibility moduli for the reconstruction
;	abs_y         : (input) visibility moduli for the (myopic convexified) data
;	inv_sigma_rad : (input) inverse of the radial standard deviation for the
;                   myopic convexified data model
;
; KEYWORD PARAMETERS:
;
;   /VERSION : (optional input) prints version number before execution.
;   
;   /HELP    : (optional input) prints the documentation and exits.
;
;
; AUTHORS:
; Serge Meimon and Laurent Mugnier (ONERA). 
; Contact: lastname at onera.fr
;
;
; RESTRICTIONS:
;   This code is copyright (c) S. Meimon and L. Mugnier, ONERA, 2004-2007. 
;   
;   WISARD is governed by the CeCILL-B license under French law and abiding by
;   the rules of distribution of free software. You can use, modify and/ or
;   redistribute the software under the terms of the CeCILL-B license as
;   circulated by CEA, CNRS and INRIA at the following URL:
;   "http://www.cecill.info".
;   See source code for the full notice.
;
;   Additionally, if you use this code or a code derived from it (e.g., for a
;   publication), please cite the following papers, on which it is based:
;   
;   (1) S. Meimon, L. M. Mugnier, and G. Le Besnerais,
;   "Reconstruction method for weak-phase optical interferometry", 
;   Opt. Lett., 30(14):1809-1811, July 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-OL-05.pdf
;   (2) S. Meimon, L. M. Mugnier, and Guy Le Besnerais,
;   "A convex approximation of the likelihood in optical interferometry",
;   J. Opt. Soc. Am. A, November 2005.
;   PDF is on-line at: http://laurent.mugnier.free.fr/publis/Meimon-JOSAA-05.pdf
;
; EXAMPLE:
;when wisard has finished and if aux_output exists, you can draw a modulus
;visibility fitting plot with:
;
;rad_freqs=aux_output.rad_freqs
;abs_hx=aux_output.weights_x.abs_hx
;abs_hx=reform(abs_hx,n_elements(abs_hx))
;abs_y=aux_output.weights_constant.abs_y_data
;abs_y=reform(abs_y,n_elements(abs_y))
;inv_sigma_rad = reform(sqrt(aux_output.cmdata.w_rad),n_elements(abs_hx))
;WISARD_PLOT_FIT,RAD_FREQS = rad_freqs, ABS_HX = abs_hx, ABS_Y = abs_y, $
;             INV_SIGMA_RAD = inv_sigma_rad, VERSION = version, HELP = help
             
; SEE ALSO:
;   All the WISARD_*.pro files, which are part of WISARD and covered by the
;   same copyright and license.
;   
; HISTORY:
;   $Log: wisard_plot_fit.pro,v $
;   Revision 1.6  2007/02/01 13:31:31  meimon
;   *** empty log message ***
;
;   Revision 1.5  2007/01/26 19:08:10  meimon
;   doc added
;
;-


PRO WISARD_PLOT_FIT,RAD_FREQS = rad_freqs, ABS_HX = abs_hx, ABS_Y = abs_y, $
             INV_SIGMA_RAD = inv_sigma_rad, VERSION = version, HELP = help

on_error,2
IF keyword_set(version) THEN $
    printf, -2, '% '+routine_courante()+': $Revision: 1.6 $, $Date: 2007/02/01 13:31:31 $'

IF (n_params() NE 0) OR keyword_set(help) THEN BEGIN ;(fill-in from syntax)
    message, 'Help required or incorrect syntax. Documentation:', /INFO
    doc_library, routine_courante()
    retall
ENDIF





xtitle =  'frequency' ; textoidl('|\nu|') 
ytitle = 'VIS.'
colors = [254, 150, 100]
plot,rad_freqs,abs_hx,psym=7,yr=[0,1], xr = [0, max(rad_freqs)*1.1], xtit = xtitle, XMARGIN=[10,4], $
     color = colors[0]        ; red
oplot,rad_freqs,abs_y,psym=6,color = colors[1] ;green
ord_rad = sort(rad_freqs)
oplot, rad_freqs[ord_rad],(0.1D*abs(abs_hx-abs_y)*inv_sigma_rad)[ord_rad], $
       color=colors[2] ;blue

; tstring = [textoidl('|y(x)|'), textoidl('|y^{data}|'), $
;            textoidl('||y(x)|-|y^{data}||/10\sigma_{rad}')]
tstring = ['Abs(Reconstructed Vis.)', 'Abs(Measured Vis.)', $
           'Abs(Difference)/(10 x stddev)']

tlinestyle= [0,0,0]
tsym= [7, 6, -3]
legend, tstring, line = tlinestyle, psym = tsym, color = colors, /top, /right, $
         clear = clear

END
